//******************************************************************************
//
//                  LCD 16X2 (CLCD) Interface Library (C File)
//
//******************************************************************************
//  FileName:           clcd.c
//  Microcontroller:    PIC18F4520
//  Compiler:           C18
//  Target Hardware:    PIC KICK Development Board
//  Company:            ENTESLA
//  website:            www.entesla.com
//  email:              feedbackATenteslaDOTcom
//
//  Author              Date            Comment
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  WebWarrior          10/04/10        Original
//  dRaGonRaCeR         13/07/10        Revision-1
//******************************************************************************

//*******INCLUDES***************************************************************
#include    "clcd.h"
#include    "config.h"
//****End of INCLUDES***********************************************************
 
//**********************Private Macro Definitions*******************************
#define _ClcdDelay45Us()    {uSDelay=PROC_FREQ_MHz*2;while(--uSDelay != 0) continue;}
#define _SET_RS()           CTRL_PORT = CTRL_PORT | (1<<CTRL_RS) 
#define _SET_EN()           CTRL_PORT = CTRL_PORT | (1<<CTRL_EN)          
#define _CLEAR_RS()         CTRL_PORT = CTRL_PORT & (~(1<<CTRL_RS))
#define _CLEAR_EN()         CTRL_PORT = CTRL_PORT & (~(1<<CTRL_EN))
#define _EnToggle()         {_SET_EN();_CLEAR_EN();}
#define _TYPE_CMD            0
#define _TYPE_DATA           1
//*******************End of Private Macro Definitions***************************

//****Variable Declarations*****************************************************
volatile unsigned char uSDelay;
//****End of Variable Declarations**********************************************

//*******Private Protoype*******************************************************
void _ClcdDelayMs(unsigned int delay);
//*****End of Private Protoype//************************************************

//**********************Preprocessor Calculation********************************
#define UPPER_NIBBLE       0xF0
#define LOWER_NIBBLE       0X0F


#if (DATA_PORT_NIBBLE == LOWER_NIBBLE)
	#define rShiftByte 4
	#define lShiftByte 0
#endif	
#if (DATA_PORT_NIBBLE == UPPER_NIBBLE)
	#define rShiftByte 0
	#define lShiftByte 4
#endif
//********************End of Preprocessor Calculation***************************

//******************************************************************************
// Function:        void ClcdInit(void)
//
// Input:           None
//
// Output:          None
//
// Overview:        CLCD initialization.
//
// Note:            None
//******************************************************************************
void ClcdInit(void)
{
    DATA_DIR &= ~DATA_PORT_NIBBLE;          //initialize D4:D7 pins as outputs
    CTRL_DIR = (CTRL_DIR & (~(1<<CTRL_RS)));//initialize RS pins as output
    CTRL_DIR = (CTRL_DIR & (~(1<<CTRL_EN)));//initialize EN pins as output   
    _CLEAR_EN();                            //clear EN
    _CLEAR_RS();                            //clear RS
    _ClcdDelayMs(300);
    ClcdSendByte(0X03,0);       //Configure bus width as 8-bit
    _ClcdDelayMs(50);
    ClcdSendByte(0X02,0);       //Configure bus width as 4-bit, 1 line,5X7 dots
    _ClcdDelayMs(50);
    ClcdSendByte(0X28,0);       //Configure bus width as 4-bit, 2 line,5X7 dots
    _ClcdDelayMs(50);
    ClcdSendByte(0X10,0);       //Cursor move and Shift to left
    _ClcdDelayMs(1);
    ClcdSendByte(0x0D,0);       //DisplayOn,CursorOff
    _ClcdDelayMs(1);
    ClcdSendByte(0x06,0);       //EntryMode,Automatic Increment-No Display shift
    _ClcdDelayMs(1);   
    ClcdSendByte(0x01,0);       //Clear Display and set address DDRAM with 0X00 
    _ClcdDelayMs(5);
}

//******************************************************************************
// Function:        void ClcdSendByte(unsigned char byte,unsigned char type)
//
// Input:           A character byte and Type of Byte (command or data)
//
// Output:          None
//
// Overview:        Send a command or a data byte to the CLCD.
//
// Note:            None
//******************************************************************************
void ClcdSendByte(unsigned char byte,unsigned char type)
{
    _CLEAR_EN();
    if(type==_TYPE_DATA)
    {
        _SET_RS();      // Selects data Register for read / write   
    }    
    else if(type==_TYPE_CMD)
    {
        _CLEAR_RS();    // Selects cmd  Register for write
    }    
    
    DATA_PORT &= ~DATA_PORT_NIBBLE;
    DATA_PORT |= (byte>>rShiftByte) & DATA_PORT_NIBBLE;                                                   
    _EnToggle();                          
    _ClcdDelay45Us();    
    
    DATA_PORT &= ~DATA_PORT_NIBBLE;
    DATA_PORT |= (byte<<lShiftByte) & DATA_PORT_NIBBLE;
    _EnToggle();        
    _ClcdDelay45Us(); 
}

//******************************************************************************
// Function:        void ClcdPutS(void *str)
//
// Input:           A character string.
//
// Output:          None
//
// Overview:        Takes a string as input and displays it on the LCD
//
// Note:            Declare an array of type char in the calling function.
//                  Declaration example: char msg0[8]="ENTESLA";
//					Usage example: ClcdPutS(msg0);
//******************************************************************************
void ClcdPutS(void *str)
{ 
    unsigned char   *str1 = (unsigned char*) str;
    while(*str1)                 		//Check for valid character 
    {                
        ClcdSendByte(*str1++,_TYPE_DATA); //Send out the current byte pointed to
    } 
}

//******************************************************************************
// Function:        void ClcdPutS_P(const rom char *str)
//
// Input:           A character string.
//
// Output:          None
//
// Overview:        Takes a string as input and displays it on the LCD
//
// Note:            example: ClcdPutS_P("ENTESLA");
//******************************************************************************
void ClcdPutS_P(const rom char *str)
{ 
    while(*str)                 		//Check for valid character 
    {                
        ClcdSendByte(*str++,_TYPE_DATA); //Send out the current byte pointed to
    } 
}

//******************************************************************************
// Function:        void ClcdGoto(unsigned char x,unsigned char y)
//
// Input:           X and Y co-ordinate.
//
// Output:          None
//
// Overview:        Moves the cursor to a position defined by x and y
//
// Note:            x = 1 to 16 for 16 column LCD.
//					y = 1 to 2 for 2 line LCD.
//******************************************************************************
void ClcdGoto(unsigned char x,unsigned char y)
{
    switch(y)
    {
        case 1:
            // position for line 1
            y=0x80 ;
            break;
            
        case 2:
            // position for line 2
            y=0xC0 ;
            break;
            
        case 3:
            // position for line 3
            y=0x94 ;
            break;
            
        case 4:
             // position for line 4
            y=0xD4 ;
            break;
            
        default:
            break;           
    }
    ClcdSendByte((x-1+y),_TYPE_CMD);	
}

//******************************************************************************
// Function:        void ClcdClear()
//
// Input:           None
//
// Output:          None
//
// Overview:        Clears the CLCD screen.
//
// Note:            None
//******************************************************************************
void ClcdClear()
{
    ClcdSendByte(0x01,_TYPE_CMD);//Clear Display and set address DDRAM with 0X00 
    _ClcdDelayMs(2);             // delay after sending clear command
}

//******************************************************************************
// Function:        void _ClcdDelayMs(unsigned int delay)
//
// Input:           Delay in milliseconds
//
// Output:          None
//
// Overview:        Generates a delay of specified number of milliseconds.
//
// Note:            None
//******************************************************************************
void _ClcdDelayMs(unsigned int delay)
{
    volatile unsigned long j;
    volatile unsigned char i;
    
    j=delay*PROC_FREQ_MHz;
    while(j--)  
    {
        for(i=0;i<18;i++); 
    }
}
 